@@ -54,7 +54,7 @@ from zope.app.publication.zopepublication import ZopePublication
 from zope.component import queryUtility, getUtility, adapter
 from zope.container.contained import Contained
 from zope.event import notify
-from zope.interface import implementer, implements, alsoProvides, noLongerProvides
+from zope.interface import implementer, implements, alsoProvides, noLongerProvides, Interface
 from zope.location import locate, Location
 from zope.schema.fieldproperty import FieldProperty
 from zope.site import hooks
@@ -202,6 +202,28 @@ class BaseTask(Persistent, Location):
                                               'job_id': self.internal_id }])
             socket.recv_json()
 
+    def launch(self):
+        scheduler = getParent(self, IScheduler)
+        # Ask scheduler for immediate execution
+        transaction.get().addAfterCommitHook(self._launchAction, kws={ 'scheduler': scheduler })
+
+    def _launchAction(self, status, *args, **kw):
+        if not status:
+            return
+        # Contact ZMQ scheduler process to ask for immediate execution
+        scheduler = kw.get('scheduler')
+        if scheduler is None:
+            scheduler = getParent(self, IScheduler)
+        if scheduler is not None:
+            zeo = getUtility(IZEOConnection, scheduler.zeo_connection)
+            context = zmq.Context()
+            socket = context.socket(zmq.REQ)
+            socket.connect('tcp://' + scheduler.zmq_address)
+            socket.send_json(['run_task', { 'zeo': zeo.getSettings(),
+                                            'task_name': traversing_api.getName(self),
+                                            'job_id': self.internal_id }])
+            socket.recv_json()
+
     def __call__(self, *args, **kw):
         report = codecs.getwriter('utf-8')(StringIO())
         self._run(report, **kw)
@@ -225,7 +247,7 @@ class BaseTask(Persistent, Location):
             try:
                 root_folder = root.get(ZopePublication.root_name, None)
                 task = traversing_api.traverse(root_folder, traversing_api.getPath(self))
-                if not task.runnable:
+                if not (kw.get('run_immediate') or task.runnable):
                     return
                 manager = ITransactionManager(task)
                 for attempt in manager.attempts():
@@ -303,7 +325,7 @@ class BaseTask(Persistent, Location):
 #
 
 class CronTaskScheduler(object):
-    """Cron-style task class"""
+    """Cron-style task scheduler class"""
 
     implements(ISchedulerTaskSchedulingMode)
 
@@ -363,7 +385,7 @@ class SchedulerTaskCronInfo(Persistent):
 #
 
 class DateTaskScheduler(object):
-    """Date-style task class"""
+    """Date-style task scheduler class"""
 
     implements(ISchedulerTaskSchedulingMode)
 
@@ -402,12 +424,21 @@ class SchedulerTaskDateInfo(Persistent):
     start_date = FieldProperty(ISchedulerDateTaskInfo['start_date'])
 
 
+#
+# Immediate-style scheduling mode
+#
+
+def ImmediateTaskTrigger():
+    """Immediate-style task scheduler class"""
+    return SimpleTrigger(run_date=datetime.now().replace(tzinfo=None) + timedelta(seconds=5))
+
+
 #
 # Loop-style scheduling mode
 #
 
 class LoopTaskScheduler(object):
-    """Interval-based task class"""
+    """Interval-based task scheduler class"""
 
     implements(ISchedulerTaskSchedulingMode)
 
